package com.gic.spark.datasource.entity;

import com.gic.spark.application.SparkEnvManager;
import com.gic.spark.util.HivePartitionUtil;
import com.gic.spark.util.SparkHiveUtil;
import com.google.common.base.Joiner;
import org.apache.spark.sql.Column;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;

import java.util.List;

/**
 * Created by paste on 2018/7/27 16:59
 * <p>
 * TODO: class description
 */
public class DataSourceHive extends DataSourceEntity {
    private String sourceTable;
    private String schema;
    private String table;

    public DataSourceHive(String sourceTable) {
        this.sourceType = DataSourceType.HIVE;
        this.sourceTable = sourceTable;

        String[] split = sourceTable.split("\\.");
        if (split.length != 2) {
            throw new RuntimeException("invalidate source table name: " + sourceTable);
        }
        schema = split[0];
        table = split[1];
    }

    @Override
    public String getSourceKey() {
        return sourceType.getName() + "_" + sourceTable;
    }

    @Override
    public String getHiveTableName() {
        return "tag." + schema.replaceAll("\\.", "_") + "_" + table;
    }


    @Override
    public void extractDataToPartitionedHiveTable(List<Integer> enterpriseList) {
        SparkSession sparkSession = SparkEnvManager.getInstance().getSparkSession();
        Dataset<Row> sourceDataset = sparkSession
                .sql(String.format("select * from %s where enterprise_id in (%s)", sourceTable, Joiner.on("','").join(enterpriseList)))
                .repartition(new Column("enterprise_id"));

        if (table.equals("new_t_fact_order_ddd_cloud")) {
            sourceDataset = sparkSession
                    .sql(String.format("select * from %s where enterprise_id in (%s) distribute by enterprise_id, is_member_order", sourceTable, Joiner.on("','").join(enterpriseList)));
        }

        SparkHiveUtil.createHivePartitionTable(sourceDataset, "enterprise_id", "tag", schema.replaceAll("\\.", "_") + "_" + table);

        HivePartitionUtil.saveDatasetToPartitionTable(sparkSession, sourceDataset, getHiveTableName());
    }
}
